@@ -1,5 +1,5 @@
 /* .eh_frame section optimization.
-   Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007
+   Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
    Free Software Foundation, Inc.
    Written by Jakub Jelinek <jakub@redhat.com>.
 
@@ -24,7 +24,7 @@
 #include "bfd.h"
 #include "libbfd.h"
 #include "elf-bfd.h"
-#include "elf/dwarf2.h"
+#include "dwarf2.h"
 
 #define EH_FRAME_HDR_SIZE 8
 
@@ -215,8 +215,8 @@ write_value (bfd *abfd, bfd_byte *buf, bfd_vma value, int width)
 static int
 cie_eq (const void *e1, const void *e2)
 {
-  const struct cie *c1 = e1;
-  const struct cie *c2 = e2;
+  const struct cie *c1 = (const struct cie *) e1;
+  const struct cie *c2 = (const struct cie *) e2;
 
   if (c1->hash == c2->hash
       && c1->length == c2->length
@@ -246,7 +246,7 @@ cie_eq (const void *e1, const void *e2)
 static hashval_t
 cie_hash (const void *e)
 {
-  const struct cie *c = e;
+  const struct cie *c = (const struct cie *) e;
   return c->hash;
 }
 
@@ -423,6 +423,28 @@ skip_non_nops (bfd_byte *buf, bfd_byte *end, unsigned int encoded_ptr_width,
   return last;
 }
 
+/* Convert absolute encoding ENCODING into PC-relative form.
+   SIZE is the size of a pointer.  */
+
+static unsigned char
+make_pc_relative (unsigned char encoding, unsigned int ptr_size)
+{
+  if ((encoding & 0x7f) == DW_EH_PE_absptr)
+    switch (ptr_size)
+      {
+      case 2:
+	encoding |= DW_EH_PE_sdata2;
+	break;
+      case 4:
+	encoding |= DW_EH_PE_sdata4;
+	break;
+      case 8:
+	encoding |= DW_EH_PE_sdata8;
+	break;
+      }
+  return encoding | DW_EH_PE_pcrel;
+}
+
 /* Called before calling _bfd_elf_parse_eh_frame on every input bfd's
    .eh_frame section.  */
 
@@ -529,26 +551,30 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
       REQUIRE (skip_bytes (&buf, end, hdr_length - 4));
     }
 
-  sec_info = bfd_zmalloc (sizeof (struct eh_frame_sec_info)
-			  + (num_entries - 1) * sizeof (struct eh_cie_fde));
+  sec_info = (struct eh_frame_sec_info *)
+      bfd_zmalloc (sizeof (struct eh_frame_sec_info)
+                   + (num_entries - 1) * sizeof (struct eh_cie_fde));
   REQUIRE (sec_info);
 
   /* We need to have a "struct cie" for each CIE in this section.  */
-  local_cies = bfd_zmalloc (num_cies * sizeof (*local_cies));
+  local_cies = (struct cie *) bfd_zmalloc (num_cies * sizeof (*local_cies));
   REQUIRE (local_cies);
 
+  /* FIXME: octets_per_byte.  */
 #define ENSURE_NO_RELOCS(buf)				\
   REQUIRE (!(cookie->rel < cookie->relend		\
 	     && (cookie->rel->r_offset			\
 		 < (bfd_size_type) ((buf) - ehbuf))	\
 	     && cookie->rel->r_info != 0))
 
+  /* FIXME: octets_per_byte.  */
 #define SKIP_RELOCS(buf)				\
   while (cookie->rel < cookie->relend			\
 	 && (cookie->rel->r_offset			\
 	     < (bfd_size_type) ((buf) - ehbuf)))	\
     cookie->rel++
 
+  /* FIXME: octets_per_byte.  */
 #define GET_RELOC(buf)					\
   ((cookie->rel < cookie->relend			\
     && (cookie->rel->r_offset				\
@@ -610,7 +636,9 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
 	  REQUIRE (read_byte (&buf, end, &cie->version));
 
 	  /* Cannot handle unknown versions.  */
-	  REQUIRE (cie->version == 1 || cie->version == 3);
+	  REQUIRE (cie->version == 1
+		   || cie->version == 3
+		   || cie->version == 4);
 	  REQUIRE (strlen ((char *) buf) < sizeof (cie->augmentation));
 
 	  strcpy (cie->augmentation, (char *) buf);
@@ -625,6 +653,13 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
 	      REQUIRE (skip_bytes (&buf, end, ptr_size));
 	      SKIP_RELOCS (buf);
 	    }
+	  if (cie->version >= 4)
+	    {
+	      REQUIRE (buf + 1 < end);
+	      REQUIRE (buf[0] == ptr_size);
+	      REQUIRE (buf[1] == 0);
+	      buf += 2;
+	    }
 	  REQUIRE (read_uleb128 (&buf, end, &cie->code_align));
 	  REQUIRE (read_sleb128 (&buf, end, &cie->data_align));
 	  if (cie->version == 1)
@@ -671,11 +706,12 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
 		      per_width = get_DW_EH_PE_width (cie->per_encoding,
 						      ptr_size);
 		      REQUIRE (per_width);
-		      if ((cie->per_encoding & 0xf0) == DW_EH_PE_aligned)
+		      if ((cie->per_encoding & 0x70) == DW_EH_PE_aligned)
 			{
 			  length = -(buf - ehbuf) & (per_width - 1);
 			  REQUIRE (skip_bytes (&buf, end, length));
 			}
+		      this_inf->u.cie.personality_offset = buf - start;
 		      ENSURE_NO_RELOCS (buf);
 		      /* Ensure we have a reloc here.  */
 		      REQUIRE (GET_RELOC (buf));
@@ -701,27 +737,23 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
 		  ->elf_backend_can_make_relative_eh_frame
 		  (abfd, info, sec)))
 	    {
-	      if ((cie->fde_encoding & 0xf0) == DW_EH_PE_absptr)
+	      if ((cie->fde_encoding & 0x70) == DW_EH_PE_absptr)
 		this_inf->make_relative = 1;
 	      /* If the CIE doesn't already have an 'R' entry, it's fairly
 		 easy to add one, provided that there's no aligned data
 		 after the augmentation string.  */
 	      else if (cie->fde_encoding == DW_EH_PE_omit
-		       && (cie->per_encoding & 0xf0) != DW_EH_PE_aligned)
+		       && (cie->per_encoding & 0x70) != DW_EH_PE_aligned)
 		{
 		  if (*cie->augmentation == 0)
 		    this_inf->add_augmentation_size = 1;
 		  this_inf->u.cie.add_fde_encoding = 1;
 		  this_inf->make_relative = 1;
 		}
-	    }
 
-	  if (info->shared
-	      && (get_elf_backend_data (abfd)
-		  ->elf_backend_can_make_lsda_relative_eh_frame
-		  (abfd, info, sec))
-	      && (cie->lsda_encoding & 0xf0) == DW_EH_PE_absptr)
-	    cie->can_make_lsda_relative = 1;
+	      if ((cie->lsda_encoding & 0x70) == DW_EH_PE_absptr)
+		cie->can_make_lsda_relative = 1;
+	    }
 
 	  /* If FDE encoding was not specified, it defaults to
 	     DW_EH_absptr.  */
@@ -766,9 +798,14 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
 
 	  /* Chain together the FDEs for each section.  */
 	  rsec = _bfd_elf_gc_mark_rsec (info, sec, gc_mark_hook, cookie);
-	  REQUIRE (rsec && rsec->owner == abfd);
-	  this_inf->u.fde.next_for_section = elf_fde_list (rsec);
-	  elf_fde_list (rsec) = this_inf;
+	  /* RSEC will be NULL if FDE was cleared out as it was belonging to
+	     a discarded SHT_GROUP.  */
+	  if (rsec)
+	    {
+	      REQUIRE (rsec->owner == abfd);
+	      this_inf->u.fde.next_for_section = elf_fde_list (rsec);
+	      elf_fde_list (rsec) = this_inf;
+	    }
 
 	  /* Skip the initial location and address range.  */
 	  start = buf;
@@ -801,6 +838,16 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
 	  insns = buf;
 
 	  buf = last_fde + 4 + hdr_length;
+
+	  /* For NULL RSEC (cleared FDE belonging to a discarded section)
+	     the relocations are commonly cleared.  We do not sanity check if
+	     all these relocations are cleared as (1) relocations to
+	     .gcc_except_table will remain uncleared (they will get dropped
+	     with the drop of this unused FDE) and (2) BFD already safely drops
+	     relocations of any type to .eh_frame by
+	     elf_section_ignore_discarded_relocs.
+	     TODO: The .gcc_except_table entries should be also filtered as
+	     .eh_frame entries; or GCC could rather use COMDAT for them.  */
 	  SKIP_RELOCS (buf);
 	}
 
@@ -824,14 +871,14 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
 	  cie->length -= end - insns_end;
 	}
       if (set_loc_count
-	  && ((cie->fde_encoding & 0xf0) == DW_EH_PE_pcrel
+	  && ((cie->fde_encoding & 0x70) == DW_EH_PE_pcrel
 	      || this_inf->make_relative))
 	{
 	  unsigned int cnt;
 	  bfd_byte *p;
 
-	  this_inf->set_loc = bfd_malloc ((set_loc_count + 1)
-					  * sizeof (unsigned int));
+	  this_inf->set_loc = (unsigned int *)
+              bfd_malloc ((set_loc_count + 1) * sizeof (unsigned int));
 	  REQUIRE (this_inf->set_loc);
 	  this_inf->set_loc[0] = set_loc_count;
 	  p = insns;
@@ -896,6 +943,7 @@ mark_entry (struct bfd_link_info *info, asection *sec,
 	    struct eh_cie_fde *ent, elf_gc_mark_hook_fn gc_mark_hook,
 	    struct elf_reloc_cookie *cookie)
 {
+  /* FIXME: octets_per_byte.  */
   for (cookie->rel = cookie->rels + ent->reloc_index;
        cookie->rel < cookie->relend
 	 && cookie->rel->r_offset < ent->offset + ent->size;
@@ -943,7 +991,7 @@ _bfd_elf_gc_mark_fdes (struct bfd_link_info *info, asection *sec,
    relocations in REL.  */
 
 static struct eh_cie_fde *
-find_merged_cie (bfd *abfd, asection *sec,
+find_merged_cie (bfd *abfd, struct bfd_link_info *info, asection *sec,
 		 struct eh_frame_hdr_info *hdr_info,
 		 struct elf_reloc_cookie *cookie,
 		 struct eh_cie_fde *cie_inf)
@@ -973,6 +1021,8 @@ find_merged_cie (bfd *abfd, asection *sec,
 
   if (cie->per_encoding != DW_EH_PE_omit)
     {
+      bfd_boolean per_binds_local;
+
       /* Work out the address of personality routine, either as an absolute
 	 value or as a symbol.  */
       rel = cookie->rels + cie->personality.reloc_index;
@@ -996,6 +1046,7 @@ find_merged_cie (bfd *abfd, asection *sec,
 	    h = (struct elf_link_hash_entry *) h->root.u.i.link;
 
 	  cie->personality.h = h;
+	  per_binds_local = SYMBOL_REFERENCES_LOCAL (info, h);
 	}
       else
 	{
@@ -1016,6 +1067,17 @@ find_merged_cie (bfd *abfd, asection *sec,
 	  cie->personality.val = (sym->st_value
 				  + sym_sec->output_offset
 				  + sym_sec->output_section->vma);
+	  per_binds_local = TRUE;
+	}
+
+      if (per_binds_local
+	  && info->shared
+	  && (cie->per_encoding & 0x70) == DW_EH_PE_absptr
+	  && (get_elf_backend_data (abfd)
+	      ->elf_backend_can_make_relative_eh_frame (abfd, info, sec)))
+	{
+	  cie_inf->u.cie.make_per_encoding_relative = 1;
+	  cie_inf->u.cie.per_encoding_relative = 1;
 	}
     }
 
@@ -1036,7 +1098,7 @@ find_merged_cie (bfd *abfd, asection *sec,
   if (new_cie == NULL)
     {
       /* Keep CIE_INF and record it in the hash table.  */
-      new_cie = malloc (sizeof (struct cie));
+      new_cie = (struct cie *) malloc (sizeof (struct cie));
       if (new_cie == NULL)
 	return cie_inf;
 
@@ -1077,17 +1139,22 @@ _bfd_elf_discard_section_eh_frame
 
   hdr_info = &elf_hash_table (info)->eh_info;
   for (ent = sec_info->entry; ent < sec_info->entry + sec_info->count; ++ent)
-    if (!ent->cie)
+    if (ent->size == 4)
+      /* There should only be one zero terminator, on the last input
+	 file supplying .eh_frame (crtend.o).  Remove any others.  */
+      ent->removed = sec->map_head.s != NULL;
+    else if (!ent->cie)
       {
 	cookie->rel = cookie->rels + ent->reloc_index;
+	/* FIXME: octets_per_byte.  */
 	BFD_ASSERT (cookie->rel < cookie->relend
 		    && cookie->rel->r_offset == ent->offset + 8);
 	if (!(*reloc_symbol_deleted_p) (ent->offset + 8, cookie))
 	  {
 	    if (info->shared
-		&& (((ent->fde_encoding & 0xf0) == DW_EH_PE_absptr
+		&& (((ent->fde_encoding & 0x70) == DW_EH_PE_absptr
 		     && ent->make_relative == 0)
-		    || (ent->fde_encoding & 0xf0) == DW_EH_PE_aligned))
+		    || (ent->fde_encoding & 0x70) == DW_EH_PE_aligned))
 	      {
 		/* If a shared library uses absolute pointers
 		   which we cannot turn into PC relative,
@@ -1100,8 +1167,8 @@ _bfd_elf_discard_section_eh_frame
 	      }
 	    ent->removed = 0;
 	    hdr_info->fde_count++;
-	    ent->u.fde.cie_inf = find_merged_cie (abfd, sec, hdr_info, cookie,
-						  ent->u.fde.cie_inf);
+	    ent->u.fde.cie_inf = find_merged_cie (abfd, info, sec, hdr_info,
+						  cookie, ent->u.fde.cie_inf);
 	  }
       }
 
@@ -1211,25 +1278,20 @@ _bfd_elf_maybe_strip_eh_frame_hdr (struct bfd_link_info *info)
 
 bfd_vma
 _bfd_elf_eh_frame_section_offset (bfd *output_bfd ATTRIBUTE_UNUSED,
-				  struct bfd_link_info *info,
+				  struct bfd_link_info *info ATTRIBUTE_UNUSED,
 				  asection *sec,
 				  bfd_vma offset)
 {
   struct eh_frame_sec_info *sec_info;
-  struct elf_link_hash_table *htab;
-  struct eh_frame_hdr_info *hdr_info;
   unsigned int lo, hi, mid;
 
   if (sec->sec_info_type != ELF_INFO_TYPE_EH_FRAME)
     return offset;
-  sec_info = elf_section_data (sec)->sec_info;
+  sec_info = (struct eh_frame_sec_info *) elf_section_data (sec)->sec_info;
 
   if (offset >= sec->rawsize)
     return offset - sec->rawsize + sec->size;
 
-  htab = elf_hash_table (info);
-  hdr_info = &htab->eh_info;
-
   lo = 0;
   hi = sec_info->count;
   mid = 0;
@@ -1251,6 +1313,14 @@ _bfd_elf_eh_frame_section_offset (bfd *output_bfd ATTRIBUTE_UNUSED,
   if (sec_info->entry[mid].removed)
     return (bfd_vma) -1;
 
+  /* If converting personality pointers to DW_EH_PE_pcrel, there will be
+     no need for run-time relocation against the personality field.  */
+  if (sec_info->entry[mid].cie
+      && sec_info->entry[mid].u.cie.make_per_encoding_relative
+      && offset == (sec_info->entry[mid].offset + 8
+		    + sec_info->entry[mid].u.cie.personality_offset))
+    return (bfd_vma) -2;
+
   /* If converting to DW_EH_PE_pcrel, there will be no need for run-time
      relocation against FDE's initial_location field.  */
   if (!sec_info->entry[mid].cie
@@ -1304,6 +1374,7 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
   struct eh_cie_fde *ent;
 
   if (sec->sec_info_type != ELF_INFO_TYPE_EH_FRAME)
+    /* FIXME: octets_per_byte.  */
     return bfd_set_section_contents (abfd, sec->output_section, contents,
 				     sec->output_offset, sec->size);
 
@@ -1311,13 +1382,13 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 	      ->elf_backend_eh_frame_address_size (abfd, sec));
   BFD_ASSERT (ptr_size != 0);
 
-  sec_info = elf_section_data (sec)->sec_info;
+  sec_info = (struct eh_frame_sec_info *) elf_section_data (sec)->sec_info;
   htab = elf_hash_table (info);
   hdr_info = &htab->eh_info;
 
   if (hdr_info->table && hdr_info->array == NULL)
-    hdr_info->array
-      = bfd_malloc (hdr_info->fde_count * sizeof(*hdr_info->array));
+    hdr_info->array = (struct eh_frame_array_ent *)
+        bfd_malloc (hdr_info->fde_count * sizeof(*hdr_info->array));
   if (hdr_info->array == NULL)
     hdr_info = NULL;
 
@@ -1409,7 +1480,7 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 		{
 		  BFD_ASSERT (action & 1);
 		  *aug++ = 'R';
-		  *buf++ = DW_EH_PE_pcrel;
+		  *buf++ = make_pc_relative (DW_EH_PE_absptr, ptr_size);
 		  action &= ~1;
 		}
 
@@ -1420,18 +1491,20 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 		    if (action & 2)
 		      {
 			BFD_ASSERT (*buf == ent->lsda_encoding);
-			*buf |= DW_EH_PE_pcrel;
+			*buf = make_pc_relative (*buf, ptr_size);
 			action &= ~2;
 		      }
 		    buf++;
 		    break;
 		  case 'P':
+		    if (ent->u.cie.make_per_encoding_relative)
+		      *buf = make_pc_relative (*buf, ptr_size);
 		    per_encoding = *buf++;
 		    per_width = get_DW_EH_PE_width (per_encoding, ptr_size);
 		    BFD_ASSERT (per_width != 0);
 		    BFD_ASSERT (((per_encoding & 0x70) == DW_EH_PE_pcrel)
 				== ent->u.cie.per_encoding_relative);
-		    if ((per_encoding & 0xf0) == DW_EH_PE_aligned)
+		    if ((per_encoding & 0x70) == DW_EH_PE_aligned)
 		      buf = (contents
 			     + ((buf - contents + per_width - 1)
 				& ~((bfd_size_type) per_width - 1)));
@@ -1441,8 +1514,15 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 
 			val = read_value (abfd, buf, per_width,
 					  get_DW_EH_PE_signed (per_encoding));
-			val += ent->offset - ent->new_offset;
-			val -= extra_string + extra_data;
+			if (ent->u.cie.make_per_encoding_relative)
+			  val -= (sec->output_section->vma
+				  + sec->output_offset
+				  + (buf - contents));
+			else
+			  {
+			    val += (bfd_vma) ent->offset - ent->new_offset;
+			    val -= extra_string + extra_data;
+			  }
 			write_value (abfd, buf, val, per_width);
 			action &= ~4;
 		      }
@@ -1452,7 +1532,7 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 		    if (action & 1)
 		      {
 			BFD_ASSERT (*buf == ent->fde_encoding);
-			*buf |= DW_EH_PE_pcrel;
+			*buf = make_pc_relative (*buf, ptr_size);
 			action &= ~1;
 		      }
 		    buf++;
@@ -1485,9 +1565,8 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 	  address = value;
 	  if (value)
 	    {
-	      switch (ent->fde_encoding & 0xf0)
+	      switch (ent->fde_encoding & 0x70)
 		{
-		case DW_EH_PE_indirect:
 		case DW_EH_PE_textrel:
 		  BFD_ASSERT (hdr_info == NULL);
 		  break;
@@ -1500,7 +1579,7 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 		  }
 		  break;
 		case DW_EH_PE_pcrel:
-		  value += ent->offset - ent->new_offset;
+		  value += (bfd_vma) ent->offset - ent->new_offset;
 		  address += (sec->output_section->vma
 			      + sec->output_offset
 			      + ent->offset + 8);
@@ -1524,7 +1603,7 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 		   + ent->new_offset);
 	    }
 
-	  if ((ent->lsda_encoding & 0xf0) == DW_EH_PE_pcrel
+	  if ((ent->lsda_encoding & 0x70) == DW_EH_PE_pcrel
 	      || cie->u.cie.make_lsda_relative)
 	    {
 	      buf += ent->lsda_offset;
@@ -1533,8 +1612,8 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 				  get_DW_EH_PE_signed (ent->lsda_encoding));
 	      if (value)
 		{
-		  if ((ent->lsda_encoding & 0xf0) == DW_EH_PE_pcrel)
-		    value += ent->offset - ent->new_offset;
+		  if ((ent->lsda_encoding & 0x70) == DW_EH_PE_pcrel)
+		    value += (bfd_vma) ent->offset - ent->new_offset;
 		  else if (cie->u.cie.make_lsda_relative)
 		    value -= (sec->output_section->vma
 			      + sec->output_offset
@@ -1554,7 +1633,7 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 	  if (ent->set_loc)
 	    {
 	      /* Adjust DW_CFA_set_loc.  */
-	      unsigned int cnt, width;
+	      unsigned int cnt;
 	      bfd_vma new_offset;
 
 	      width = get_DW_EH_PE_width (ent->fde_encoding, ptr_size);
@@ -1564,7 +1643,6 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 
 	      for (cnt = 1; cnt <= ent->set_loc[0]; cnt++)
 		{
-		  bfd_vma value;
 		  buf = start + ent->set_loc[cnt];
 
 		  value = read_value (abfd, buf, width,
@@ -1572,8 +1650,8 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 		  if (!value)
 		    continue;
 
-		  if ((ent->fde_encoding & 0xf0) == DW_EH_PE_pcrel)
-		    value += ent->offset + 8 - new_offset;
+		  if ((ent->fde_encoding & 0x70) == DW_EH_PE_pcrel)
+		    value += (bfd_vma) ent->offset + 8 - new_offset;
 		  if (ent->make_relative)
 		    value -= (sec->output_section->vma
 			      + sec->output_offset
@@ -1592,6 +1670,7 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
   if ((sec->size % ptr_size) != 0)
     abort ();
 
+  /* FIXME: octets_per_byte.  */
   return bfd_set_section_contents (abfd, sec->output_section,
 				   contents, (file_ptr) sec->output_offset,
 				   sec->size);
@@ -1603,8 +1682,8 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
 static int
 vma_compare (const void *a, const void *b)
 {
-  const struct eh_frame_array_ent *p = a;
-  const struct eh_frame_array_ent *q = b;
+  const struct eh_frame_array_ent *p = (const struct eh_frame_array_ent *) a;
+  const struct eh_frame_array_ent *q = (const struct eh_frame_array_ent *) b;
   if (p->initial_loc > q->initial_loc)
     return 1;
   if (p->initial_loc < q->initial_loc)
@@ -1655,7 +1734,7 @@ _bfd_elf_write_section_eh_frame_hdr (bfd *abfd, struct bfd_link_info *info)
   size = EH_FRAME_HDR_SIZE;
   if (hdr_info->array && hdr_info->array_count == hdr_info->fde_count)
     size += 4 + hdr_info->fde_count * 8;
-  contents = bfd_malloc (size);
+  contents = (bfd_byte *) bfd_malloc (size);
   if (contents == NULL)
     return FALSE;
 
@@ -1703,6 +1782,7 @@ _bfd_elf_write_section_eh_frame_hdr (bfd *abfd, struct bfd_link_info *info)
 	}
     }
 
+  /* FIXME: octets_per_byte.  */
   retval = bfd_set_section_contents (abfd, sec->output_section,
 				     contents, (file_ptr) sec->output_offset,
 				     sec->size);
