@@ -1,12 +1,12 @@
 /* PEF support for BFD.
-   Copyright 1999, 2000, 2001, 2002, 2003, 2004
+   Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,17 +16,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
 
+#include "sysdep.h"
 #include "safe-ctype.h"
-
 #include "pef.h"
 #include "pef-traceback.h"
-
 #include "bfd.h"
-#include "sysdep.h"
 #include "libbfd.h"
-
 #include "libiberty.h"
 
 #ifndef BFD_IO_FUNCS
@@ -40,12 +38,10 @@
 #define bfd_pef_bfd_is_target_special_symbol ((bfd_boolean (*) (bfd *, asymbol *)) bfd_false)
 #define bfd_pef_get_lineno                          _bfd_nosymbols_get_lineno
 #define bfd_pef_find_nearest_line                   _bfd_nosymbols_find_nearest_line
+#define bfd_pef_find_inliner_info                   _bfd_nosymbols_find_inliner_info
 #define bfd_pef_bfd_make_debug_symbol               _bfd_nosymbols_bfd_make_debug_symbol
 #define bfd_pef_read_minisymbols                    _bfd_generic_read_minisymbols
 #define bfd_pef_minisymbol_to_symbol                _bfd_generic_minisymbol_to_symbol
-#define bfd_pef_get_reloc_upper_bound               _bfd_norelocs_get_reloc_upper_bound
-#define bfd_pef_canonicalize_reloc                  _bfd_norelocs_canonicalize_reloc
-#define bfd_pef_bfd_reloc_type_lookup               _bfd_norelocs_bfd_reloc_type_lookup
 #define bfd_pef_set_arch_mach                       _bfd_generic_set_arch_mach
 #define bfd_pef_get_section_contents                _bfd_generic_get_section_contents
 #define bfd_pef_set_section_contents                _bfd_generic_set_section_contents
@@ -55,105 +51,26 @@
 #define bfd_pef_bfd_merge_sections                  bfd_generic_merge_sections
 #define bfd_pef_bfd_is_group_section		    bfd_generic_is_group_section
 #define bfd_pef_bfd_discard_group                   bfd_generic_discard_group
-#define bfd_pef_section_already_linked	    _bfd_generic_section_already_linked
+#define bfd_pef_section_already_linked	            _bfd_generic_section_already_linked
+#define bfd_pef_bfd_define_common_symbol            bfd_generic_define_common_symbol
 #define bfd_pef_bfd_link_hash_table_create          _bfd_generic_link_hash_table_create
 #define bfd_pef_bfd_link_hash_table_free            _bfd_generic_link_hash_table_free
 #define bfd_pef_bfd_link_add_symbols                _bfd_generic_link_add_symbols
 #define bfd_pef_bfd_link_just_syms                  _bfd_generic_link_just_syms
+#define bfd_pef_bfd_copy_link_hash_symbol_type \
+  _bfd_generic_copy_link_hash_symbol_type
 #define bfd_pef_bfd_final_link                      _bfd_generic_final_link
 #define bfd_pef_bfd_link_split_section              _bfd_generic_link_split_section
 #define bfd_pef_get_section_contents_in_window      _bfd_generic_get_section_contents_in_window
 
-static void bfd_pef_print_symbol               PARAMS ((bfd *, PTR, asymbol *, bfd_print_symbol_type));
-static void bfd_pef_convert_architecture       PARAMS ((unsigned long, enum bfd_architecture *, unsigned long *));
-static bfd_boolean bfd_pef_mkobject            PARAMS ((bfd *));
-static int bfd_pef_parse_traceback_table       PARAMS ((bfd *, asection *, unsigned char *, size_t, size_t, asymbol *, FILE *));
-static const char *bfd_pef_section_name        PARAMS ((bfd_pef_section *));
-static unsigned long bfd_pef_section_flags     PARAMS ((bfd_pef_section *));
-static asection *bfd_pef_make_bfd_section      PARAMS ((bfd *, bfd_pef_section *));
-static int bfd_pef_read_header                 PARAMS ((bfd *, bfd_pef_header *));
-static const bfd_target *bfd_pef_object_p      PARAMS ((bfd *));
-static int bfd_pef_parse_traceback_tables      PARAMS ((bfd *, asection *, unsigned char *, size_t, long *, asymbol **));
-static int bfd_pef_parse_function_stub         PARAMS ((bfd *, unsigned char *, size_t, unsigned long *));
-static int bfd_pef_parse_function_stubs        PARAMS ((bfd *, asection *, unsigned char *, size_t, unsigned char *, size_t, unsigned long *, asymbol **));
-static long bfd_pef_parse_symbols              PARAMS ((bfd *, asymbol **));
-static long bfd_pef_count_symbols              PARAMS ((bfd *));
-static long bfd_pef_get_symtab_upper_bound     PARAMS ((bfd *));
-static long bfd_pef_canonicalize_symtab        PARAMS ((bfd *, asymbol **));
-static asymbol *bfd_pef_make_empty_symbol      PARAMS ((bfd *));
-static void bfd_pef_get_symbol_info            PARAMS ((bfd *, asymbol *, symbol_info *));
-static int bfd_pef_sizeof_headers              PARAMS ((bfd *, bfd_boolean));
-static int bfd_pef_xlib_read_header            PARAMS ((bfd *, bfd_pef_xlib_header *));
-static int bfd_pef_xlib_scan                   PARAMS ((bfd *, bfd_pef_xlib_header *));
-static const bfd_target *bfd_pef_xlib_object_p PARAMS ((bfd *));
-
-static void
-bfd_pef_print_symbol (abfd, afile, symbol, how)
-     bfd *abfd;
-     PTR afile;
-     asymbol *symbol;
-     bfd_print_symbol_type how;
-{
-  FILE *file = (FILE *) afile;
-
-  switch (how)
-    {
-    case bfd_print_symbol_name:
-      fprintf (file, "%s", symbol->name);
-      break;
-    default:
-      bfd_print_symbol_vandf (abfd, (PTR) file, symbol);
-      fprintf (file, " %-5s %s", symbol->section->name, symbol->name);
-      if (strncmp (symbol->name, "__traceback_", strlen ("__traceback_")) == 0)
-	{
-	  char *buf = alloca (symbol->udata.i);
-	  size_t offset = symbol->value + 4;
-	  size_t len = symbol->udata.i;
-	  int ret;
-
-	  bfd_get_section_contents (abfd, symbol->section, buf, offset, len);
-	  ret = bfd_pef_parse_traceback_table (abfd, symbol->section, buf,
-					       len, 0, NULL, file);
-	  if (ret < 0)
-	    fprintf (file, " [ERROR]");
-	}
-    }
-}
-
-static void
-bfd_pef_convert_architecture (architecture, type, subtype)
-     unsigned long architecture;
-     enum bfd_architecture *type;
-     unsigned long *subtype;
-{
-  const unsigned long ARCH_POWERPC = 0x70777063; /* 'pwpc' */
-  const unsigned long ARCH_M68K = 0x6d36386b; /* 'm68k' */
-
-  *subtype = bfd_arch_unknown;
-  *type = bfd_arch_unknown;
-
-  if (architecture == ARCH_POWERPC)
-    *type = bfd_arch_powerpc;
-  else if (architecture == ARCH_M68K)
-    *type = bfd_arch_m68k;
-}
-
-static bfd_boolean
-bfd_pef_mkobject (abfd)
-     bfd *abfd ATTRIBUTE_UNUSED;
-{
-  return TRUE;
-}
-
 static int
-bfd_pef_parse_traceback_table (abfd, section, buf, len, pos, sym, file)
-     bfd *abfd;
-     asection *section;
-     unsigned char *buf;
-     size_t len;
-     size_t pos;
-     asymbol *sym;
-     FILE *file;
+bfd_pef_parse_traceback_table (bfd *abfd,
+			       asection *section,
+			       unsigned char *buf,
+			       size_t len,
+			       size_t pos,
+			       asymbol *sym,
+			       FILE *file)
 {
   struct traceback_table table;
   size_t offset;
@@ -161,7 +78,7 @@ bfd_pef_parse_traceback_table (abfd, section, buf, len, pos, sym, file)
   asymbol tmpsymbol;
 
   if (sym == NULL)
-    sym = &tmpsymbol;
+    sym = & tmpsymbol;
 
   sym->name = NULL;
   sym->value = 0;
@@ -171,21 +88,19 @@ bfd_pef_parse_traceback_table (abfd, section, buf, len, pos, sym, file)
   sym->udata.i = 0;
 
   /* memcpy is fine since all fields are unsigned char.  */
-
   if ((pos + 8) > len)
     return -1;
   memcpy (&table, buf + pos, 8);
 
   /* Calling code relies on returned symbols having a name and
      correct offset.  */
-
   if ((table.lang != TB_C) && (table.lang != TB_CPLUSPLUS))
     return -1;
 
   if (! (table.flags2 & TB_NAME_PRESENT))
     return -1;
 
-  if (! table.flags1 & TB_HAS_TBOFF)
+  if (! (table.flags1 & TB_HAS_TBOFF))
     return -1;
 
   offset = 8;
@@ -204,7 +119,6 @@ bfd_pef_parse_traceback_table (abfd, section, buf, len, pos, sym, file)
 
       /* Need to subtract 4 because the offset includes the 0x0L
 	 preceding the table.  */
-
       if (file != NULL)
 	fprintf (file, " [offset = 0x%lx]", off.tb_offset);
 
@@ -248,7 +162,7 @@ bfd_pef_parse_traceback_table (abfd, section, buf, len, pos, sym, file)
       if ((pos + offset + name.name_len) > len)
 	return -1;
 
-      namebuf = (char *) bfd_alloc (abfd, name.name_len + 1);
+      namebuf = bfd_alloc (abfd, name.name_len + 1);
       if (namebuf == NULL)
 	return -1;
 
@@ -275,13 +189,67 @@ bfd_pef_parse_traceback_table (abfd, section, buf, len, pos, sym, file)
     offset += 4;
 
   if (file != NULL)
-    fprintf (file, " [length = 0x%lx]", (long) offset);
+    fprintf (file, " [length = 0x%lx]", (unsigned long) offset);
 
   return offset;
 }
 
-static const char *bfd_pef_section_name (section)
-     bfd_pef_section *section;
+static void
+bfd_pef_print_symbol (bfd *abfd,
+		      void * afile,
+		      asymbol *symbol,
+		      bfd_print_symbol_type how)
+{
+  FILE *file = (FILE *) afile;
+
+  switch (how)
+    {
+    case bfd_print_symbol_name:
+      fprintf (file, "%s", symbol->name);
+      break;
+    default:
+      bfd_print_symbol_vandf (abfd, (void *) file, symbol);
+      fprintf (file, " %-5s %s", symbol->section->name, symbol->name);
+      if (CONST_STRNEQ (symbol->name, "__traceback_"))
+	{
+	  unsigned char *buf = alloca (symbol->udata.i);
+	  size_t offset = symbol->value + 4;
+	  size_t len = symbol->udata.i;
+	  int ret;
+
+	  bfd_get_section_contents (abfd, symbol->section, buf, offset, len);
+	  ret = bfd_pef_parse_traceback_table (abfd, symbol->section, buf,
+					       len, 0, NULL, file);
+	  if (ret < 0)
+	    fprintf (file, " [ERROR]");
+	}
+    }
+}
+
+static void
+bfd_pef_convert_architecture (unsigned long architecture,
+			      enum bfd_architecture *type,
+			      unsigned long *subtype)
+{
+  const unsigned long ARCH_POWERPC = 0x70777063; /* 'pwpc'.  */
+  const unsigned long ARCH_M68K = 0x6d36386b; /* 'm68k'.  */
+
+  *subtype = bfd_arch_unknown;
+  *type = bfd_arch_unknown;
+
+  if (architecture == ARCH_POWERPC)
+    *type = bfd_arch_powerpc;
+  else if (architecture == ARCH_M68K)
+    *type = bfd_arch_m68k;
+}
+
+static bfd_boolean
+bfd_pef_mkobject (bfd *abfd ATTRIBUTE_UNUSED)
+{
+  return TRUE;
+}
+
+static const char *bfd_pef_section_name (bfd_pef_section *section)
 {
   switch (section->section_kind)
     {
@@ -298,8 +266,7 @@ static const char *bfd_pef_section_name (section)
     }
 }
 
-static unsigned long bfd_pef_section_flags (section)
-     bfd_pef_section *section;
+static unsigned long bfd_pef_section_flags (bfd_pef_section *section)
 {
   switch (section->section_kind)
     {
@@ -319,9 +286,7 @@ static unsigned long bfd_pef_section_flags (section)
 }
 
 static asection *
-bfd_pef_make_bfd_section (abfd, section)
-     bfd *abfd;
-     bfd_pef_section *section;
+bfd_pef_make_bfd_section (bfd *abfd, bfd_pef_section *section)
 {
   asection *bfdsec;
   const char *name = bfd_pef_section_name (section);
@@ -341,11 +306,11 @@ bfd_pef_make_bfd_section (abfd, section)
   return bfdsec;
 }
 
-int bfd_pef_parse_loader_header (abfd, buf, len, header)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     unsigned char *buf;
-     size_t len;
-     bfd_pef_loader_header *header;
+int
+bfd_pef_parse_loader_header (bfd *abfd ATTRIBUTE_UNUSED,
+			     unsigned char *buf,
+			     size_t len,
+			     bfd_pef_loader_header *header)
 {
   BFD_ASSERT (len == 56);
 
@@ -367,11 +332,11 @@ int bfd_pef_parse_loader_header (abfd, buf, len, header)
   return 0;
 }
 
-int bfd_pef_parse_imported_library (abfd, buf, len, header)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     unsigned char *buf;
-     size_t len;
-     bfd_pef_imported_library *header;
+int
+bfd_pef_parse_imported_library (bfd *abfd ATTRIBUTE_UNUSED,
+				unsigned char *buf,
+				size_t len,
+				bfd_pef_imported_library *header)
 {
   BFD_ASSERT (len == 24);
 
@@ -387,31 +352,30 @@ int bfd_pef_parse_imported_library (abfd, buf, len, header)
   return 0;
 }
 
-int bfd_pef_parse_imported_symbol (abfd, buf, len, symbol)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     unsigned char *buf;
-     size_t len;
-     bfd_pef_imported_symbol *symbol;
+int
+bfd_pef_parse_imported_symbol (bfd *abfd ATTRIBUTE_UNUSED,
+			       unsigned char *buf,
+			       size_t len,
+			       bfd_pef_imported_symbol *symbol)
 {
   unsigned long value;
 
   BFD_ASSERT (len == 4);
 
   value = bfd_getb32 (buf);
-  symbol->class = value >> 24;
+  symbol->symbol_class = value >> 24;
   symbol->name = value & 0x00ffffff;
 
   return 0;
 }
 
-int bfd_pef_scan_section (abfd, section)
-     bfd *abfd;
-     bfd_pef_section *section;
+int
+bfd_pef_scan_section (bfd *abfd, bfd_pef_section *section)
 {
   unsigned char buf[28];
 
   bfd_seek (abfd, section->header_offset, SEEK_SET);
-  if (bfd_bread ((PTR) buf, 28, abfd) != 28)
+  if (bfd_bread ((void *) buf, 28, abfd) != 28)
     return -1;
 
   section->name_offset = bfd_h_get_32 (abfd, buf);
@@ -433,10 +397,9 @@ int bfd_pef_scan_section (abfd, section)
 }
 
 void
-bfd_pef_print_loader_header (abfd, header, file)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     bfd_pef_loader_header *header;
-     FILE *file;
+bfd_pef_print_loader_header (bfd *abfd ATTRIBUTE_UNUSED,
+			     bfd_pef_loader_header *header,
+			     FILE *file)
 {
   fprintf (file, "main_section: %ld\n", header->main_section);
   fprintf (file, "main_offset: %lu\n", header->main_offset);
@@ -460,40 +423,24 @@ bfd_pef_print_loader_header (abfd, header, file)
 }
 
 int
-bfd_pef_print_loader_section (abfd, file)
-     bfd *abfd;
-     FILE *file;
+bfd_pef_print_loader_section (bfd *abfd, FILE *file)
 {
   bfd_pef_loader_header header;
   asection *loadersec = NULL;
   unsigned char *loaderbuf = NULL;
   size_t loaderlen = 0;
-  int ret;
 
   loadersec = bfd_get_section_by_name (abfd, "loader");
   if (loadersec == NULL)
     return -1;
 
   loaderlen = loadersec->size;
-  loaderbuf = (unsigned char *) bfd_malloc (loaderlen);
-  if (bfd_seek (abfd, loadersec->filepos, SEEK_SET) < 0)
-    {
-      free (loaderbuf);
-      return -1;
-    }
-  if (bfd_bread ((PTR) loaderbuf, loaderlen, abfd) != loaderlen)
-    {
-      free (loaderbuf);
-      return -1;
-    }
+  loaderbuf = bfd_malloc (loaderlen);
 
-  if (loaderlen < 56)
-    {
-      free (loaderbuf);
-      return -1;
-    }
-  ret = bfd_pef_parse_loader_header (abfd, loaderbuf, 56, &header);
-  if (ret < 0)
+  if (bfd_seek (abfd, loadersec->filepos, SEEK_SET) < 0
+      || bfd_bread ((void *) loaderbuf, loaderlen, abfd) != loaderlen
+      || loaderlen < 56
+      || bfd_pef_parse_loader_header (abfd, loaderbuf, 56, &header) < 0)
     {
       free (loaderbuf);
       return -1;
@@ -504,8 +451,7 @@ bfd_pef_print_loader_section (abfd, file)
 }
 
 int
-bfd_pef_scan_start_address (abfd)
-     bfd *abfd;
+bfd_pef_scan_start_address (bfd *abfd)
 {
   bfd_pef_loader_header header;
   asection *section;
@@ -520,10 +466,10 @@ bfd_pef_scan_start_address (abfd)
     goto end;
 
   loaderlen = loadersec->size;
-  loaderbuf = (unsigned char *) bfd_malloc (loaderlen);
+  loaderbuf = bfd_malloc (loaderlen);
   if (bfd_seek (abfd, loadersec->filepos, SEEK_SET) < 0)
     goto error;
-  if (bfd_bread ((PTR) loaderbuf, loaderlen, abfd) != loaderlen)
+  if (bfd_bread ((void *) loaderbuf, loaderlen, abfd) != loaderlen)
     goto error;
 
   if (loaderlen < 56)
@@ -583,9 +529,7 @@ bfd_pef_scan (abfd, header, mdata)
 
   if (header->section_count != 0)
     {
-      mdata->sections =
-	((bfd_pef_section *)
-	 bfd_alloc (abfd, header->section_count * sizeof (bfd_pef_section)));
+      mdata->sections = bfd_alloc (abfd, header->section_count * sizeof (bfd_pef_section));
 
       if (mdata->sections == NULL)
 	return -1;
@@ -600,13 +544,7 @@ bfd_pef_scan (abfd, header, mdata)
     }
 
   if (bfd_pef_scan_start_address (abfd) < 0)
-    {
-#if 0
-      fprintf (stderr, "bfd_pef_scan: unable to scan start address: %s\n",
-	       bfd_errmsg (bfd_get_error ()));
-      return -1;
-#endif
-    }
+    return -1;
 
   abfd->tdata.pef_data = mdata;
 
@@ -614,15 +552,13 @@ bfd_pef_scan (abfd, header, mdata)
 }
 
 static int
-bfd_pef_read_header (abfd, header)
-     bfd *abfd;
-     bfd_pef_header *header;
+bfd_pef_read_header (bfd *abfd, bfd_pef_header *header)
 {
   unsigned char buf[40];
 
   bfd_seek (abfd, 0, SEEK_SET);
 
-  if (bfd_bread ((PTR) buf, 40, abfd) != 40)
+  if (bfd_bread ((void *) buf, 40, abfd) != 40)
     return -1;
 
   header->tag1 = bfd_getb32 (buf);
@@ -641,8 +577,7 @@ bfd_pef_read_header (abfd, header)
 }
 
 static const bfd_target *
-bfd_pef_object_p (abfd)
-     bfd *abfd;
+bfd_pef_object_p (bfd *abfd)
 {
   struct bfd_preserve preserve;
   bfd_pef_header header;
@@ -675,13 +610,13 @@ bfd_pef_object_p (abfd)
   return NULL;
 }
 
-static int bfd_pef_parse_traceback_tables (abfd, sec, buf, len, nsym, csym)
-     bfd *abfd;
-     asection *sec;
-     unsigned char *buf;
-     size_t len;
-     long *nsym;
-     asymbol **csym;
+static int
+bfd_pef_parse_traceback_tables (bfd *abfd,
+				asection *sec,
+				unsigned char *buf,
+				size_t len,
+				long *nsym,
+				asymbol **csym)
 {
   char *name;
 
@@ -727,14 +662,13 @@ static int bfd_pef_parse_traceback_tables (abfd, sec, buf, len, nsym, csym)
 
       /* Don't bother to compute the name if we are just
 	 counting symbols.  */
-
       if (csym)
 	{
 	  tbnamelen = strlen (tbprefix) + strlen (function.name);
 	  name = bfd_alloc (abfd, tbnamelen + 1);
 	  if (name == NULL)
 	    {
-	      bfd_release (abfd, (PTR) function.name);
+	      bfd_release (abfd, (void *) function.name);
 	      function.name = NULL;
 	      break;
 	    }
@@ -758,11 +692,11 @@ static int bfd_pef_parse_traceback_tables (abfd, sec, buf, len, nsym, csym)
   return 0;
 }
 
-static int bfd_pef_parse_function_stub (abfd, buf, len, offset)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     unsigned char *buf;
-     size_t len;
-     unsigned long *offset;
+static int
+bfd_pef_parse_function_stub (bfd *abfd ATTRIBUTE_UNUSED,
+			     unsigned char *buf,
+			     size_t len,
+			     unsigned long *offset)
 {
   BFD_ASSERT (len == 24);
 
@@ -785,26 +719,22 @@ static int bfd_pef_parse_function_stub (abfd, buf, len, offset)
   return 0;
 }
 
-static int bfd_pef_parse_function_stubs (abfd, codesec, codebuf, codelen,
-					 loaderbuf, loaderlen, nsym, csym)
-     bfd *abfd;
-     asection *codesec;
-     unsigned char *codebuf;
-     size_t codelen;
-     unsigned char *loaderbuf;
-     size_t loaderlen;
-     unsigned long *nsym;
-     asymbol **csym;
+static int
+bfd_pef_parse_function_stubs (bfd *abfd,
+			      asection *codesec,
+			      unsigned char *codebuf,
+			      size_t codelen,
+			      unsigned char *loaderbuf,
+			      size_t loaderlen,
+			      unsigned long *nsym,
+			      asymbol **csym)
 {
   const char *const sprefix = "__stub_";
-
   size_t codepos = 0;
   unsigned long count = 0;
-
   bfd_pef_loader_header header;
   bfd_pef_imported_library *libraries = NULL;
   bfd_pef_imported_symbol *imports = NULL;
-
   unsigned long i;
   int ret;
 
@@ -815,9 +745,9 @@ static int bfd_pef_parse_function_stubs (abfd, codesec, codebuf, codelen,
   if (ret < 0)
     goto error;
 
-  libraries = (bfd_pef_imported_library *) bfd_malloc
+  libraries = bfd_malloc
     (header.imported_library_count * sizeof (bfd_pef_imported_library));
-  imports = (bfd_pef_imported_symbol *) bfd_malloc
+  imports = bfd_malloc
     (header.total_imported_symbol_count * sizeof (bfd_pef_imported_symbol));
 
   if (loaderlen < (56 + (header.imported_library_count * 24)))
@@ -850,8 +780,7 @@ static int bfd_pef_parse_function_stubs (abfd, codesec, codebuf, codelen,
       asymbol sym;
       const char *symname;
       char *name;
-      unsigned long index;
-      int ret;
+      unsigned long sym_index;
 
       if (csym && (csym[count] == NULL))
 	break;
@@ -869,14 +798,14 @@ static int bfd_pef_parse_function_stubs (abfd, codesec, codebuf, codelen,
       if ((codepos + 4) > codelen)
 	break;
 
-      ret = bfd_pef_parse_function_stub (abfd, codebuf + codepos, 24, &index);
+      ret = bfd_pef_parse_function_stub (abfd, codebuf + codepos, 24, &sym_index);
       if (ret < 0)
 	{
 	  codepos += 24;
 	  continue;
 	}
 
-      if (index >= header.total_imported_symbol_count)
+      if (sym_index >= header.total_imported_symbol_count)
 	{
 	  codepos += 24;
 	  continue;
@@ -886,11 +815,12 @@ static int bfd_pef_parse_function_stubs (abfd, codesec, codebuf, codelen,
 	size_t max, namelen;
 	const char *s;
 
-	if (loaderlen < (header.loader_strings_offset + imports[index].name))
+	if (loaderlen < (header.loader_strings_offset + imports[sym_index].name))
 	  goto error;
 
-	max = loaderlen - (header.loader_strings_offset + imports[index].name);
-	symname = loaderbuf + header.loader_strings_offset + imports[index].name;
+	max = loaderlen - (header.loader_strings_offset + imports[sym_index].name);
+	symname = (char *) loaderbuf;
+	symname += header.loader_strings_offset + imports[sym_index].name;
 	namelen = 0;
 	for (s = symname; s < (symname + max); s++)
 	  {
@@ -945,9 +875,8 @@ static int bfd_pef_parse_function_stubs (abfd, codesec, codebuf, codelen,
   return -1;
 }
 
-static long bfd_pef_parse_symbols (abfd, csym)
-     bfd *abfd;
-     asymbol **csym;
+static long
+bfd_pef_parse_symbols (bfd *abfd, asymbol **csym)
 {
   unsigned long count = 0;
 
@@ -963,10 +892,10 @@ static long bfd_pef_parse_symbols (abfd, csym)
   if (codesec != NULL)
     {
       codelen = codesec->size;
-      codebuf = (unsigned char *) bfd_malloc (codelen);
+      codebuf = bfd_malloc (codelen);
       if (bfd_seek (abfd, codesec->filepos, SEEK_SET) < 0)
 	goto end;
-      if (bfd_bread ((PTR) codebuf, codelen, abfd) != codelen)
+      if (bfd_bread ((void *) codebuf, codelen, abfd) != codelen)
 	goto end;
     }
 
@@ -974,17 +903,17 @@ static long bfd_pef_parse_symbols (abfd, csym)
   if (loadersec != NULL)
     {
       loaderlen = loadersec->size;
-      loaderbuf = (unsigned char *) bfd_malloc (loaderlen);
+      loaderbuf = bfd_malloc (loaderlen);
       if (bfd_seek (abfd, loadersec->filepos, SEEK_SET) < 0)
 	goto end;
-      if (bfd_bread ((PTR) loaderbuf, loaderlen, abfd) != loaderlen)
+      if (bfd_bread ((void *) loaderbuf, loaderlen, abfd) != loaderlen)
 	goto end;
     }
 
   count = 0;
   if (codesec != NULL)
     {
-      unsigned long ncount = 0;
+      long ncount = 0;
       bfd_pef_parse_traceback_tables (abfd, codesec, codebuf, codelen,
 				      &ncount, csym);
       count += ncount;
@@ -1013,32 +942,29 @@ static long bfd_pef_parse_symbols (abfd, csym)
 }
 
 static long
-bfd_pef_count_symbols (abfd)
-     bfd *abfd;
+bfd_pef_count_symbols (bfd *abfd)
 {
   return bfd_pef_parse_symbols (abfd, NULL);
 }
 
 static long
-bfd_pef_get_symtab_upper_bound (abfd)
-     bfd *abfd;
+bfd_pef_get_symtab_upper_bound (bfd *abfd)
 {
   long nsyms = bfd_pef_count_symbols (abfd);
+
   if (nsyms < 0)
     return nsyms;
   return ((nsyms + 1) * sizeof (asymbol *));
 }
 
 static long
-bfd_pef_canonicalize_symtab (abfd, alocation)
-     bfd *abfd;
-     asymbol **alocation;
+bfd_pef_canonicalize_symtab (bfd *abfd, asymbol **alocation)
 {
   long i;
   asymbol *syms;
   long ret;
-
   long nsyms = bfd_pef_count_symbols (abfd);
+
   if (nsyms < 0)
     return nsyms;
 
@@ -1058,63 +984,56 @@ bfd_pef_canonicalize_symtab (abfd, alocation)
   return ret;
 }
 
-static asymbol *
-bfd_pef_make_empty_symbol (abfd)
-     bfd *abfd;
-{
-  return (asymbol *) bfd_alloc (abfd, sizeof (asymbol));
-}
+#define bfd_pef_make_empty_symbol _bfd_generic_make_empty_symbol
 
 static void
-bfd_pef_get_symbol_info (abfd, symbol, ret)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     asymbol *symbol;
-     symbol_info *ret;
+bfd_pef_get_symbol_info (bfd *abfd ATTRIBUTE_UNUSED,
+			 asymbol *symbol,
+			 symbol_info *ret)
 {
   bfd_symbol_info (symbol, ret);
 }
 
 static int
-bfd_pef_sizeof_headers (abfd, exec)
-     bfd *abfd ATTRIBUTE_UNUSED;
-     bfd_boolean exec ATTRIBUTE_UNUSED;
+bfd_pef_sizeof_headers (bfd *abfd ATTRIBUTE_UNUSED,
+			struct bfd_link_info *info ATTRIBUTE_UNUSED)
 {
   return 0;
 }
 
 const bfd_target pef_vec =
 {
-  "pef",			/* name */
-  bfd_target_pef_flavour,	/* flavour */
-  BFD_ENDIAN_BIG,		/* byteorder */
-  BFD_ENDIAN_BIG,		/* header_byteorder */
-  (HAS_RELOC | EXEC_P |		/* object flags */
+  "pef",			/* Name.  */
+  bfd_target_pef_flavour,	/* Flavour.  */
+  BFD_ENDIAN_BIG,		/* Byteorder.  */
+  BFD_ENDIAN_BIG,		/* Header_byteorder.  */
+  (HAS_RELOC | EXEC_P |		/* Object flags.  */
    HAS_LINENO | HAS_DEBUG |
    HAS_SYMS | HAS_LOCALS | DYNAMIC | WP_TEXT | D_PAGED),
   (SEC_ALLOC | SEC_LOAD | SEC_READONLY | SEC_CODE | SEC_DATA
-   | SEC_ROM | SEC_HAS_CONTENTS), /* section_flags */
-  0,				/* symbol_leading_char */
-  ' ',				/* ar_pad_char */
-  16,				/* ar_max_namelen */
+   | SEC_ROM | SEC_HAS_CONTENTS), /* Section_flags.  */
+  0,				/* Symbol_leading_char.  */
+  ' ',				/* AR_pad_char.  */
+  16,				/* AR_max_namelen.  */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
-  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* data */
+  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Data.  */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
-  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* hdrs */
-  {				/* bfd_check_format */
+  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Headers.  */
+  {				/* bfd_check_format.  */
     _bfd_dummy_target,
-    bfd_pef_object_p,		/* bfd_check_format */
+    bfd_pef_object_p,		/* bfd_check_format.  */
     _bfd_dummy_target,
     _bfd_dummy_target,
   },
-  {				/* bfd_set_format */
+  {				/* bfd_set_format.  */
     bfd_false,
     bfd_pef_mkobject,
     bfd_false,
     bfd_false,
   },
-  {				/* bfd_write_contents */
+  {				/* bfd_write_contents.  */
     bfd_false,
     bfd_true,
     bfd_false,
@@ -1126,7 +1045,7 @@ const bfd_target pef_vec =
   BFD_JUMP_TABLE_CORE (_bfd_nocore),
   BFD_JUMP_TABLE_ARCHIVE (_bfd_noarchive),
   BFD_JUMP_TABLE_SYMBOLS (bfd_pef),
-  BFD_JUMP_TABLE_RELOCS (bfd_pef),
+  BFD_JUMP_TABLE_RELOCS (_bfd_norelocs),
   BFD_JUMP_TABLE_WRITE (bfd_pef),
   BFD_JUMP_TABLE_LINK (bfd_pef),
   BFD_JUMP_TABLE_DYNAMIC (_bfd_nodynamic),
@@ -1136,24 +1055,22 @@ const bfd_target pef_vec =
   NULL
 };
 
-#define bfd_pef_xlib_close_and_cleanup _bfd_generic_close_and_cleanup
-#define bfd_pef_xlib_bfd_free_cached_info _bfd_generic_bfd_free_cached_info
-#define bfd_pef_xlib_new_section_hook _bfd_generic_new_section_hook
-#define bfd_pef_xlib_get_section_contents _bfd_generic_get_section_contents
-#define bfd_pef_xlib_set_section_contents _bfd_generic_set_section_contents
+#define bfd_pef_xlib_close_and_cleanup              _bfd_generic_close_and_cleanup
+#define bfd_pef_xlib_bfd_free_cached_info           _bfd_generic_bfd_free_cached_info
+#define bfd_pef_xlib_new_section_hook               _bfd_generic_new_section_hook
+#define bfd_pef_xlib_get_section_contents           _bfd_generic_get_section_contents
+#define bfd_pef_xlib_set_section_contents           _bfd_generic_set_section_contents
 #define bfd_pef_xlib_get_section_contents_in_window _bfd_generic_get_section_contents_in_window
 #define bfd_pef_xlib_set_section_contents_in_window _bfd_generic_set_section_contents_in_window
 
 static int
-bfd_pef_xlib_read_header (abfd, header)
-     bfd *abfd;
-     bfd_pef_xlib_header *header;
+bfd_pef_xlib_read_header (bfd *abfd, bfd_pef_xlib_header *header)
 {
   unsigned char buf[76];
 
   bfd_seek (abfd, 0, SEEK_SET);
 
-  if (bfd_bread ((PTR) buf, 76, abfd) != 76)
+  if (bfd_bread ((void *) buf, 76, abfd) != 76)
     return -1;
 
   header->tag1 = bfd_getb32 (buf);
@@ -1180,15 +1097,12 @@ bfd_pef_xlib_read_header (abfd, header)
   return 0;
 }
 
-int
-bfd_pef_xlib_scan (abfd, header)
-     bfd *abfd;
-     bfd_pef_xlib_header *header;
+static int
+bfd_pef_xlib_scan (bfd *abfd, bfd_pef_xlib_header *header)
 {
   bfd_pef_xlib_data_struct *mdata = NULL;
 
-  mdata = ((bfd_pef_xlib_data_struct *)
-	   bfd_alloc (abfd, sizeof (bfd_pef_xlib_data_struct)));
+  mdata = bfd_alloc (abfd, sizeof (* mdata));
   if (mdata == NULL)
     return -1;
 
@@ -1203,8 +1117,7 @@ bfd_pef_xlib_scan (abfd, header)
 }
 
 static const bfd_target *
-bfd_pef_xlib_object_p (abfd)
-     bfd *abfd;
+bfd_pef_xlib_object_p (bfd *abfd)
 {
   struct bfd_preserve preserve;
   bfd_pef_xlib_header header;
@@ -1242,37 +1155,37 @@ bfd_pef_xlib_object_p (abfd)
 
 const bfd_target pef_xlib_vec =
 {
-  "pef-xlib",			/* name */
-  bfd_target_pef_xlib_flavour,	/* flavour */
-  BFD_ENDIAN_BIG,		/* byteorder */
-  BFD_ENDIAN_BIG,		/* header_byteorder */
-  (HAS_RELOC | EXEC_P |		/* object flags */
+  "pef-xlib",			/* Name.  */
+  bfd_target_pef_xlib_flavour,	/* Flavour.  */
+  BFD_ENDIAN_BIG,		/* Byteorder */
+  BFD_ENDIAN_BIG,		/* Header_byteorder.  */
+  (HAS_RELOC | EXEC_P |		/* Object flags.  */
    HAS_LINENO | HAS_DEBUG |
    HAS_SYMS | HAS_LOCALS | DYNAMIC | WP_TEXT | D_PAGED),
   (SEC_ALLOC | SEC_LOAD | SEC_READONLY | SEC_CODE | SEC_DATA
-   | SEC_ROM | SEC_HAS_CONTENTS), /* section_flags */
-  0,				/* symbol_leading_char */
-  ' ',				/* ar_pad_char */
-  16,				/* ar_max_namelen */
+   | SEC_ROM | SEC_HAS_CONTENTS),/* Section_flags.  */
+  0,				/* Symbol_leading_char.  */
+  ' ',				/* AR_pad_char.  */
+  16,				/* AR_max_namelen.  */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
-  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* data */
+  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Data.  */
   bfd_getb64, bfd_getb_signed_64, bfd_putb64,
   bfd_getb32, bfd_getb_signed_32, bfd_putb32,
-  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* hdrs */
-  {				/* bfd_check_format */
+  bfd_getb16, bfd_getb_signed_16, bfd_putb16,	/* Headers.  */
+  {				/* bfd_check_format.  */
     _bfd_dummy_target,
-    bfd_pef_xlib_object_p,	/* bfd_check_format */
+    bfd_pef_xlib_object_p,	/* bfd_check_format.  */
     _bfd_dummy_target,
     _bfd_dummy_target,
   },
-  {				/* bfd_set_format */
+  {				/* bfd_set_format.  */
     bfd_false,
     bfd_pef_mkobject,
     bfd_false,
     bfd_false,
   },
-  {				/* bfd_write_contents */
+  {				/* bfd_write_contents.  */
     bfd_false,
     bfd_true,
     bfd_false,
