@@ -88,13 +88,6 @@ typedef enum
   }
 queue_position;
 
-/* Tell create_file_handler what events we are interested in. 
-   This is used by the select version of the event loop. */
-
-#define GDB_READABLE	(1<<1)
-#define GDB_WRITABLE	(1<<2)
-#define GDB_EXCEPTION	(1<<3)
-
 /* Exported functions from event-loop.c */
 
 extern void start_event_loop (void);
