@@ -1,7 +1,8 @@
 /* Print in infix form a struct expression.
 
    Copyright (C) 1986, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-   1998, 1999, 2000, 2003, 2007, 2008, 2009 Free Software Foundation, Inc.
+   1998, 1999, 2000, 2003, 2007, 2008, 2009, 2010
+   Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -41,6 +42,7 @@ void
 print_expression (struct expression *exp, struct ui_file *stream)
 {
   int pc = 0;
+
   print_subexp (exp, &pc, stream, PREC_NULL);
 }
 
@@ -95,6 +97,7 @@ print_subexp_standard (struct expression *exp, int *pos,
     case OP_LONG:
       {
 	struct value_print_options opts;
+
 	get_raw_print_options (&opts);
 	(*pos) += 3;
 	value_print (value_from_longest (exp->elts[pc + 1].type,
@@ -106,6 +109,7 @@ print_subexp_standard (struct expression *exp, int *pos,
     case OP_DOUBLE:
       {
 	struct value_print_options opts;
+
 	get_raw_print_options (&opts);
 	(*pos) += 3;
 	value_print (value_from_double (exp->elts[pc + 1].type,
@@ -117,6 +121,7 @@ print_subexp_standard (struct expression *exp, int *pos,
     case OP_VAR_VALUE:
       {
 	struct block *b;
+
 	(*pos) += 3;
 	b = exp->elts[pc + 1].block;
 	if (b != NULL
@@ -139,6 +144,7 @@ print_subexp_standard (struct expression *exp, int *pos,
     case OP_REGISTER:
       {
 	const char *name = &exp->elts[pc + 2].string;
+
 	(*pos) += 3 + BYTES_TO_EXP_ELEM (exp->elts[pc + 1].longconst + 1);
 	fprintf_filtered (stream, "$%s", name);
 	return;
@@ -180,6 +186,7 @@ print_subexp_standard (struct expression *exp, int *pos,
     case OP_STRING:
       {
 	struct value_print_options opts;
+
 	nargs = longest_to_int (exp->elts[pc + 1].longconst);
 	(*pos) += 3 + BYTES_TO_EXP_ELEM (nargs + 1);
 	/* LA_PRINT_STRING will print using the current repeat count threshold.
@@ -187,7 +194,7 @@ print_subexp_standard (struct expression *exp, int *pos,
 	   additional parameter to LA_PRINT_STRING.  -fnf */
 	get_user_print_options (&opts);
 	LA_PRINT_STRING (stream, builtin_type (exp->gdbarch)->builtin_char,
-			 &exp->elts[pc + 2].string, nargs, 0, &opts);
+			 &exp->elts[pc + 2].string, nargs, NULL, 0, &opts);
       }
       return;
 
@@ -201,12 +208,13 @@ print_subexp_standard (struct expression *exp, int *pos,
     case OP_OBJC_NSSTRING:	/* Objective-C Foundation Class NSString constant.  */
       {
 	struct value_print_options opts;
+
 	nargs = longest_to_int (exp->elts[pc + 1].longconst);
 	(*pos) += 3 + BYTES_TO_EXP_ELEM (nargs + 1);
 	fputs_filtered ("@\"", stream);
 	get_user_print_options (&opts);
 	LA_PRINT_STRING (stream, builtin_type (exp->gdbarch)->builtin_char,
-			 &exp->elts[pc + 2].string, nargs, 0, &opts);
+			 &exp->elts[pc + 2].string, nargs, NULL, 0, &opts);
 	fputs_filtered ("\"", stream);
       }
       return;
@@ -214,6 +222,7 @@ print_subexp_standard (struct expression *exp, int *pos,
     case OP_OBJC_MSGCALL:
       {			/* Objective C message (method) call.  */
 	char *selector;
+
 	(*pos) += 3;
 	nargs = longest_to_int (exp->elts[pc + 2].longconst);
 	fprintf_unfiltered (stream, "[");
@@ -227,6 +236,7 @@ print_subexp_standard (struct expression *exp, int *pos,
 	if (nargs)
 	  {
 	    char *s, *nextS;
+
 	    s = alloca (strlen (selector) + 1);
 	    strcpy (s, selector);
 	    for (tem = 0; tem < nargs; tem++)
@@ -290,9 +300,10 @@ print_subexp_standard (struct expression *exp, int *pos,
       if (tem > 0)
 	{
 	  struct value_print_options opts;
+
 	  get_user_print_options (&opts);
 	  LA_PRINT_STRING (stream, builtin_type (exp->gdbarch)->builtin_char,
-			   tempstr, nargs - 1, 0, &opts);
+			   tempstr, nargs - 1, NULL, 0, &opts);
 	  (*pos) = pc;
 	}
       else
@@ -409,12 +420,24 @@ print_subexp_standard (struct expression *exp, int *pos,
 	fputs_filtered (")", stream);
       return;
 
+    case UNOP_DYNAMIC_CAST:
+    case UNOP_REINTERPRET_CAST:
+      fputs_filtered (opcode == UNOP_DYNAMIC_CAST ? "dynamic_cast"
+		      : "reinterpret_cast", stream);
+      fputs_filtered ("<", stream);
+      (*pos) += 2;
+      type_print (exp->elts[pc + 1].type, "", stream, 0);
+      fputs_filtered ("> (", stream);
+      print_subexp (exp, pos, stream, PREC_PREFIX);
+      fputs_filtered (")", stream);
+      return;
+
     case UNOP_MEMVAL:
       (*pos) += 2;
       if ((int) prec > (int) PREC_PREFIX)
 	fputs_filtered ("(", stream);
-      if (TYPE_CODE (exp->elts[pc + 1].type) == TYPE_CODE_FUNC &&
-	  exp->elts[pc + 3].opcode == OP_LONG)
+      if (TYPE_CODE (exp->elts[pc + 1].type) == TYPE_CODE_FUNC
+	  && exp->elts[pc + 3].opcode == OP_LONG)
 	{
 	  struct value_print_options opts;
 
@@ -509,10 +532,6 @@ print_subexp_standard (struct expression *exp, int *pos,
       fprintf_unfiltered (stream, ")");
       return;
 
-    case BINOP_INCL:
-    case BINOP_EXCL:
-      error (_("print_subexp:  Not implemented."));
-
       /* Default ops */
 
     default:
@@ -691,10 +710,6 @@ op_name_standard (enum exp_opcode opcode)
       return "BINOP_ASSIGN_MODIFY";
     case BINOP_VAL:
       return "BINOP_VAL";
-    case BINOP_INCL:
-      return "BINOP_INCL";
-    case BINOP_EXCL:
-      return "BINOP_EXCL";
     case BINOP_CONCAT:
       return "BINOP_CONCAT";
     case BINOP_RANGE:
@@ -729,6 +744,10 @@ op_name_standard (enum exp_opcode opcode)
       return "OP_ARRAY";
     case UNOP_CAST:
       return "UNOP_CAST";
+    case UNOP_DYNAMIC_CAST:
+      return "UNOP_DYNAMIC_CAST";
+    case UNOP_REINTERPRET_CAST:
+      return "UNOP_REINTERPRET_CAST";
     case UNOP_MEMVAL:
       return "UNOP_MEMVAL";
     case UNOP_MEMVAL_TLS:
@@ -753,12 +772,6 @@ op_name_standard (enum exp_opcode opcode)
       return "UNOP_POSTDECREMENT";
     case UNOP_SIZEOF:
       return "UNOP_SIZEOF";
-    case UNOP_LOWER:
-      return "UNOP_LOWER";
-    case UNOP_UPPER:
-      return "UNOP_UPPER";
-    case UNOP_LENGTH:
-      return "UNOP_LENGTH";
     case UNOP_PLUS:
       return "UNOP_PLUS";
     case UNOP_CAP:
@@ -799,6 +812,8 @@ op_name_standard (enum exp_opcode opcode)
       return "OP_TYPE";
     case OP_LABELED:
       return "OP_LABELED";
+    case OP_ADL_FUNC:
+      return "OP_ADL_FUNC";
     }
 }
 
@@ -923,8 +938,6 @@ dump_subexp_body_standard (struct expression *exp,
     case BINOP_INTDIV:
     case BINOP_ASSIGN_MODIFY:
     case BINOP_VAL:
-    case BINOP_INCL:
-    case BINOP_EXCL:
     case BINOP_CONCAT:
     case BINOP_IN:
     case BINOP_RANGE:
@@ -953,12 +966,6 @@ dump_subexp_body_standard (struct expression *exp,
     case UNOP_MIN:
     case UNOP_ODD:
     case UNOP_TRUNC:
-    case UNOP_LOWER:
-    case UNOP_UPPER:
-    case UNOP_LENGTH:
-    case UNOP_CARD:
-    case UNOP_CHMAX:
-    case UNOP_CHMIN:
       elt = dump_subexp (exp, stream, elt);
       break;
     case OP_LONG:
@@ -1035,6 +1042,8 @@ dump_subexp_body_standard (struct expression *exp,
       break;
     case UNOP_MEMVAL:
     case UNOP_CAST:
+    case UNOP_DYNAMIC_CAST:
+    case UNOP_REINTERPRET_CAST:
       fprintf_filtered (stream, "Type @");
       gdb_print_host_address (exp->elts[elt].type, stream);
       fprintf_filtered (stream, " (");
