@@ -1,6 +1,6 @@
 /* MI Command Set - environment commands.
 
-   Copyright (C) 2002, 2003, 2004, 2007, 2008, 2009
+   Copyright (C) 2002, 2003, 2004, 2007, 2008, 2009, 2010
    Free Software Foundation, Inc.
 
    Contributed by Red Hat Inc.
@@ -52,6 +52,7 @@ env_execute_cli_command (const char *cmd, const char *args)
     {
       struct cleanup *old_cleanups;
       char *run;
+
       if (args != NULL)
 	run = xstrprintf ("%s %s", cmd, args);
       else
@@ -141,6 +142,7 @@ mi_cmd_env_path (char *command, char **argv, int argc)
     {
       int opt = mi_getopt ("mi_cmd_env_path", argc, argv, opts,
                            &optind, &optarg);
+
       if (opt < 0)
         break;
       switch ((enum opt) opt)
@@ -162,7 +164,7 @@ mi_cmd_env_path (char *command, char **argv, int argc)
   else
     {
       /* Otherwise, get current path to modify.  */
-      env = get_in_environ (inferior_environ, path_var_name);
+      env = get_in_environ (current_inferior ()->environment, path_var_name);
 
       /* Can be null if path is not set.  */
       if (!env)
@@ -173,9 +175,9 @@ mi_cmd_env_path (char *command, char **argv, int argc)
   for (i = argc - 1; i >= 0; --i)
     env_mod_path (argv[i], &exec_path);
 
-  set_in_environ (inferior_environ, path_var_name, exec_path);
+  set_in_environ (current_inferior ()->environment, path_var_name, exec_path);
   xfree (exec_path);
-  env = get_in_environ (inferior_environ, path_var_name);
+  env = get_in_environ (current_inferior ()->environment, path_var_name);
   ui_out_field_string (uiout, "path", env);
 }
 
@@ -211,6 +213,7 @@ mi_cmd_env_dir (char *command, char **argv, int argc)
     {
       int opt = mi_getopt ("mi_cmd_env_dir", argc, argv, opts,
                            &optind, &optarg);
+
       if (opt < 0)
         break;
       switch ((enum opt) opt)
@@ -260,10 +263,17 @@ mi_cmd_inferior_tty_show (char *command, char **argv, int argc)
 void 
 _initialize_mi_cmd_env (void)
 {
+  struct gdb_environ *environment;
   char *env;
 
-  /* We want original execution path to reset to, if desired later.  */
-  env = get_in_environ (inferior_environ, path_var_name);
+  /* We want original execution path to reset to, if desired later.
+     At this point, current inferior is not created, so cannot use
+     current_inferior ()->environment.  Also, there's no obvious
+     place where this code can be moved suchs that it surely run
+     before any code possibly mangles original PATH.  */
+  environment = make_environ ();
+  init_environ (environment);
+  env = get_in_environ (environment, path_var_name);
 
   /* Can be null if path is not set.  */
   if (!env)
