@@ -1,7 +1,7 @@
 /* IBM RS/6000 native-dependent code for GDB, the GNU debugger.
 
    Copyright (C) 1986, 1987, 1989, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-   1998, 1999, 2000, 2001, 2002, 2003, 2004, 2007, 2008
+   1998, 1999, 2000, 2001, 2002, 2003, 2004, 2007, 2008, 2009, 2010
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -37,6 +37,7 @@
 #include "rs6000-tdep.h"
 #include "exec.h"
 #include "observer.h"
+#include "xcoffread.h"
 
 #include <sys/ptrace.h>
 #include <sys/reg.h>
@@ -74,7 +75,7 @@
 #ifndef ARCH3264
 # define ARCH64() 0
 #else
-# define ARCH64() (register_size (current_gdbarch, 0) == 8)
+# define ARCH64() (register_size (target_gdbarch, 0) == 8)
 #endif
 
 /* Union of 32-bit and 64-bit versions of ld_info. */
@@ -129,7 +130,7 @@ static int objfile_symbol_add (void *);
 
 static void vmap_symtab (struct vmap *);
 
-static void exec_one_dummy_insn (struct gdbarch *);
+static void exec_one_dummy_insn (struct regcache *);
 
 extern void fixup_breakpoints (CORE_ADDR low, CORE_ADDR high, CORE_ADDR delta);
 
@@ -199,8 +200,8 @@ rs6000_ptrace64 (int req, int id, long long addr, int data, void *buf)
   int ret = 0;
 #endif
 #if 0
-  printf ("rs6000_ptrace64 (%d, %d, 0x%llx, %08x, 0x%x) = 0x%x\n",
-	  req, id, addr, data, (unsigned int)buf, ret);
+  printf ("rs6000_ptrace64 (%d, %d, %s, %08x, 0x%x) = 0x%x\n",
+	  req, id, hex_string (addr), data, (unsigned int)buf, ret);
 #endif
   return ret;
 }
@@ -266,7 +267,7 @@ fetch_register (struct regcache *regcache, int regno)
 /* Store register REGNO back into the inferior. */
 
 static void
-store_register (const struct regcache *regcache, int regno)
+store_register (struct regcache *regcache, int regno)
 {
   struct gdbarch *gdbarch = get_regcache_arch (regcache);
   int addr[MAX_REGISTER_SIZE];
@@ -302,7 +303,7 @@ store_register (const struct regcache *regcache, int regno)
 	   Otherwise the following ptrace(2) calls will mess up user stack
 	   since kernel will get confused about the bottom of the stack
 	   (%sp). */
-	exec_one_dummy_insn (gdbarch);
+	exec_one_dummy_insn (regcache);
 
       /* The PT_WRITE_GPR operation is rather odd.  For 32-bit inferiors,
          the register's value is passed by value, but for 64-bit inferiors,
@@ -333,7 +334,8 @@ store_register (const struct regcache *regcache, int regno)
    REGNO otherwise. */
 
 static void
-rs6000_fetch_inferior_registers (struct regcache *regcache, int regno)
+rs6000_fetch_inferior_registers (struct target_ops *ops,
+				 struct regcache *regcache, int regno)
 {
   struct gdbarch *gdbarch = get_regcache_arch (regcache);
   if (regno != -1)
@@ -375,7 +377,8 @@ rs6000_fetch_inferior_registers (struct regcache *regcache, int regno)
    Otherwise, REGNO specifies which register (so we can save time).  */
 
 static void
-rs6000_store_inferior_registers (struct regcache *regcache, int regno)
+rs6000_store_inferior_registers (struct target_ops *ops,
+				 struct regcache *regcache, int regno)
 {
   struct gdbarch *gdbarch = get_regcache_arch (regcache);
   if (regno != -1)
@@ -517,7 +520,8 @@ rs6000_xfer_partial (struct target_ops *ops, enum target_object object,
    the status in *OURSTATUS.  */
 
 static ptid_t
-rs6000_wait (ptid_t ptid, struct target_waitstatus *ourstatus)
+rs6000_wait (struct target_ops *ops,
+	     ptid_t ptid, struct target_waitstatus *ourstatus, int options)
 {
   pid_t pid;
   int status, save_errno;
@@ -525,7 +529,6 @@ rs6000_wait (ptid_t ptid, struct target_waitstatus *ourstatus)
   do
     {
       set_sigint_trap ();
-      set_sigio_trap ();
 
       do
 	{
@@ -534,7 +537,6 @@ rs6000_wait (ptid_t ptid, struct target_waitstatus *ourstatus)
 	}
       while (pid == -1 && errno == EINTR);
 
-      clear_sigio_trap ();
       clear_sigint_trap ();
 
       if (pid == -1)
@@ -546,7 +548,7 @@ rs6000_wait (ptid_t ptid, struct target_waitstatus *ourstatus)
 	  /* Claim it exited with unknown signal.  */
 	  ourstatus->kind = TARGET_WAITKIND_SIGNALLED;
 	  ourstatus->value.sig = TARGET_SIGNAL_UNKNOWN;
-	  return minus_one_ptid;
+	  return inferior_ptid;
 	}
 
       /* Ignore terminated detached child processes.  */
@@ -575,10 +577,11 @@ rs6000_wait (ptid_t ptid, struct target_waitstatus *ourstatus)
    including u_area. */
 
 static void
-exec_one_dummy_insn (struct gdbarch *gdbarch)
+exec_one_dummy_insn (struct regcache *regcache)
 {
 #define	DUMMY_INSN_ADDR	AIX_TEXT_SEGMENT_BASE+0x200
 
+  struct gdbarch *gdbarch = get_regcache_arch (regcache);
   int ret, status, pid;
   CORE_ADDR prev_pc;
   void *bp;
@@ -587,15 +590,15 @@ exec_one_dummy_insn (struct gdbarch *gdbarch)
      assume that this address will never be executed again by the real
      code. */
 
-  bp = deprecated_insert_raw_breakpoint (DUMMY_INSN_ADDR);
+  bp = deprecated_insert_raw_breakpoint (gdbarch, NULL, DUMMY_INSN_ADDR);
 
   /* You might think this could be done with a single ptrace call, and
      you'd be correct for just about every platform I've ever worked
      on.  However, rs6000-ibm-aix4.1.3 seems to have screwed this up --
      the inferior never hits the breakpoint (it's also worth noting
      powerpc-ibm-aix4.1.3 works correctly).  */
-  prev_pc = read_pc ();
-  write_pc (DUMMY_INSN_ADDR);
+  prev_pc = regcache_read_pc (regcache);
+  regcache_write_pc (regcache, DUMMY_INSN_ADDR);
   if (ARCH64 ())
     ret = rs6000_ptrace64 (PT_CONTINUE, PIDGET (inferior_ptid), 1, 0, NULL);
   else
@@ -610,8 +613,8 @@ exec_one_dummy_insn (struct gdbarch *gdbarch)
     }
   while (pid != PIDGET (inferior_ptid));
 
-  write_pc (prev_pc);
-  deprecated_remove_raw_breakpoint (bp);
+  regcache_write_pc (regcache, prev_pc);
+  deprecated_remove_raw_breakpoint (gdbarch, bp);
 }
 
 
@@ -689,8 +692,8 @@ objfile_symbol_add (void *arg)
 {
   struct objfile *obj = (struct objfile *) arg;
 
-  syms_from_objfile (obj, NULL, 0, 0, 0, 0);
-  new_symfile_objfile (obj, 0, 0);
+  syms_from_objfile (obj, NULL, 0, 0, 0);
+  new_symfile_objfile (obj, 0);
   return 1;
 }
 
@@ -734,8 +737,8 @@ add_vmap (LdInfo *ldi)
 
   filename = LDI_FILENAME (ldi, arch64);
   mem = filename + strlen (filename) + 1;
-  mem = savestring (mem, strlen (mem));
-  objname = savestring (filename, strlen (filename));
+  mem = xstrdup (mem);
+  objname = xstrdup (filename);
 
   fd = LDI_FD (ldi, arch64);
   if (fd < 0)
@@ -912,7 +915,7 @@ If in fact that file has symbols which the mapped files listed by\n\
 symbols to the proper address)."),
 	       symfile_objfile->name);
       free_objfile (symfile_objfile);
-      symfile_objfile = NULL;
+      gdb_assert (symfile_objfile == NULL);
     }
   breakpoint_re_set ();
 }
@@ -932,33 +935,32 @@ vmap_exec (void)
 {
   static bfd *execbfd;
   int i;
+  struct target_section_table *table = target_get_section_table (&exec_ops);
 
   if (execbfd == exec_bfd)
     return;
 
   execbfd = exec_bfd;
 
-  if (!vmap || !exec_ops.to_sections)
-    error (_("vmap_exec: vmap or exec_ops.to_sections == 0."));
+  if (!vmap || !table->sections)
+    error (_("vmap_exec: vmap or table->sections == 0."));
 
-  for (i = 0; &exec_ops.to_sections[i] < exec_ops.to_sections_end; i++)
+  for (i = 0; &table->sections[i] < table->sections_end; i++)
     {
-      if (strcmp (".text", exec_ops.to_sections[i].the_bfd_section->name) == 0)
+      if (strcmp (".text", table->sections[i].the_bfd_section->name) == 0)
 	{
-	  exec_ops.to_sections[i].addr += vmap->tstart - vmap->tvma;
-	  exec_ops.to_sections[i].endaddr += vmap->tstart - vmap->tvma;
+	  table->sections[i].addr += vmap->tstart - vmap->tvma;
+	  table->sections[i].endaddr += vmap->tstart - vmap->tvma;
 	}
-      else if (strcmp (".data",
-		       exec_ops.to_sections[i].the_bfd_section->name) == 0)
+      else if (strcmp (".data", table->sections[i].the_bfd_section->name) == 0)
 	{
-	  exec_ops.to_sections[i].addr += vmap->dstart - vmap->dvma;
-	  exec_ops.to_sections[i].endaddr += vmap->dstart - vmap->dvma;
+	  table->sections[i].addr += vmap->dstart - vmap->dvma;
+	  table->sections[i].endaddr += vmap->dstart - vmap->dvma;
 	}
-      else if (strcmp (".bss",
-		       exec_ops.to_sections[i].the_bfd_section->name) == 0)
+      else if (strcmp (".bss", table->sections[i].the_bfd_section->name) == 0)
 	{
-	  exec_ops.to_sections[i].addr += vmap->dstart - vmap->dvma;
-	  exec_ops.to_sections[i].endaddr += vmap->dstart - vmap->dvma;
+	  table->sections[i].addr += vmap->dstart - vmap->dvma;
+	  table->sections[i].endaddr += vmap->dstart - vmap->dvma;
 	}
     }
 }
@@ -966,17 +968,18 @@ vmap_exec (void)
 /* Set the current architecture from the host running GDB.  Called when
    starting a child process. */
 
-static void (*super_create_inferior) (char *exec_file, char *allargs,
-				      char **env, int from_tty);
+static void (*super_create_inferior) (struct target_ops *,char *exec_file, 
+				      char *allargs, char **env, int from_tty);
 static void
-rs6000_create_inferior (char *exec_file, char *allargs, char **env, int from_tty)
+rs6000_create_inferior (struct target_ops * ops, char *exec_file,
+			char *allargs, char **env, int from_tty)
 {
   enum bfd_architecture arch;
   unsigned long mach;
   bfd abfd;
   struct gdbarch_info info;
 
-  super_create_inferior (exec_file, allargs, env, from_tty);
+  super_create_inferior (ops, exec_file, allargs, env, from_tty);
 
   if (__power_rs ())
     {
@@ -1033,7 +1036,8 @@ xcoff_relocate_symtab (unsigned int pid)
   int ldisize = arch64 ? sizeof (ldi->l64) : sizeof (ldi->l32);
   int size;
 
-  if (ptid_equal (inferior_ptid, null_ptid))
+  /* Nothing to do if we are debugging a core file.  */
+  if (!target_has_execution)
     return;
 
   do
@@ -1154,10 +1158,9 @@ xcoff_relocate_core (struct target_ops *target)
          add our sections to the section table for the core target.  */
       if (vp != vmap)
 	{
-	  struct section_table *stp;
+	  struct target_section *stp;
 
-	  target_resize_to_sections (target, 2);
-	  stp = target->to_sections_end - 2;
+	  stp = deprecated_core_resize_section_table (2);
 
 	  stp->bfd = vp->bfd;
 	  stp->the_bfd_section = bfd_get_section_by_name (stp->bfd, ".text");
@@ -1191,16 +1194,15 @@ static CORE_ADDR
 find_toc_address (CORE_ADDR pc)
 {
   struct vmap *vp;
-  extern CORE_ADDR get_toc_offset (struct objfile *);	/* xcoffread.c */
 
   for (vp = vmap; vp; vp = vp->nxt)
     {
       if (pc >= vp->tstart && pc < vp->tend)
 	{
 	  /* vp->objfile is only NULL for the exec file.  */
-	  return vp->dstart + get_toc_offset (vp->objfile == NULL
-					      ? symfile_objfile
-					      : vp->objfile);
+	  return vp->dstart + xcoff_get_toc_offset (vp->objfile == NULL
+						    ? symfile_objfile
+						    : vp->objfile);
 	}
     }
   error (_("Unable to find TOC entry for pc %s."), hex_string (pc));
