@@ -2,6 +2,16 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
+
+#ifdef SIGNALS
+#include <signal.h>
+
+static void
+sigint_handler (int signo)
+{
+}
+#endif
+
 int
 main ()
 {
@@ -10,6 +20,9 @@ main ()
 #ifdef usestubs
   set_debug_traps();
   breakpoint();
+#endif
+#ifdef SIGNALS
+  signal (SIGINT, sigint_handler);
 #endif
   printf ("talk to me baby\n");
   while (1)
@@ -20,7 +33,10 @@ main ()
 #ifdef EINTR
 	  if (errno != EINTR)
 #endif
-	    perror ("");
+	    {
+	      perror ("");
+	      return 1;
+	    }
 	}
       else if (nbytes == 0)
 	{
