@@ -1,24 +1,23 @@
-/* This file is part of psim (model of the PowerPC(tm) architecture)
+/* The common simulator framework for GDB, the GNU Debugger.
 
-   Copyright (C) 1994-1997, Andrew Cagney <cagney@highland.com.au>
+   Copyright 2002, 2005, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License
-   as published by the Free Software Foundation; either version 2 of
-   the License, or (at your option) any later version.
- 
-   This library is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
- 
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- 
-   --
+   Contributed by Andrew Cagney and Red Hat.
 
-   PowerPC is a trademark of International Business Machines Corporation. */
+   This file is part of GDB.
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 #ifndef SIM_TYPES_H
@@ -111,11 +110,11 @@ typedef struct { signed64 a[2]; } signed128;
 typedef signed char signed8;
 typedef signed short signed16;
 #if defined (__ALPHA__)
-typedef signed int unsigned32;
-typedef signed long unsigned64;
+typedef signed int signed32;
+typedef signed long signed64;
 #else
-typedef signed long unsigned32;
-typedef signed long long unsigned64;
+typedef signed long signed32;
+typedef signed long long signed64;
 #endif
 
 typedef unsigned char unsigned8;
